/**
 * \addtogroup DS1307
 * @{
 */

/*****************************************************************************/
/**
 * \file DS1307.c
 * DS1307 real-time clock routines for the 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 01-05-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#include "DS1307.h"

/*****************************************************************************
 *
 *                                  Structures
 *
 *****************************************************************************/
struct ds1307_st
{
    unsigned char seconds;
    unsigned char minutes;
    unsigned char hours;
    unsigned char day;
    unsigned char date;
    unsigned char month;
    unsigned char year;
};


/*****************************************************************************
 *
 *                      Private Global Variables Definitions  
 *
 *****************************************************************************/
struct ds1307_st buff;

const char * code am_pm_as_text[] = {"AM", "PM"};

const char * code day_as_text[] 
                  = {"void", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};

const char * code month_as_text[] = {"void", "Jan", "Feb", "Mar", "Apr",
                      "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};


/*****************************************************************************
 *
 *                         Private Function Prototypes
 *
 *****************************************************************************/
static unsigned char ds1307_BCD_to_hex(unsigned char byte);
static unsigned char ds1307_hex_to_BCD(unsigned char byte);


/*****************************************************************************
 *
 *                       Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                             ds1307_BCD_to_hex()
 *
 *****************************************************************************/
static unsigned char ds1307_BCD_to_hex(unsigned char byte)
{
    return((((byte & 0xF0) >> 4) * 10) + (byte & 0x0F));
}


/*****************************************************************************
 *
 *                             ds1307_hex_to_BCD()
 *
 *****************************************************************************/
static unsigned char ds1307_hex_to_BCD(unsigned char byte)
{
    return((((byte / 10) & 0x0F) << 4) | (byte % 10));
}


/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                                 ds1307_init()
 *
 *****************************************************************************/
void ds1307_init(unsigned char mode)
{
    /* Send software reset sequence so as to I2C-protocol-reset all slaves. */
    i2c_transmit_reset();

    /* Retrieve current time from RTC and store in local buffer. */
    ds1307_get_buff_from_rtc();

    /* Adjust clock halt bit 7 in buffered seconds register to enable */
    /* (CH = 0) or disable (CH = 1) the realtime clock's oscillator. */
    buff.seconds &= 0x7F;
    buff.seconds |= (mode & DS1307_CLOCK_HALT) << 4;

    /* Adjust 12-hour mode bit 6 in buffered hours register to select */
    /* 12-hour (bit 6 = 1) or 24-hour (bit 6 = 0) clock operating mode. */
    buff.hours &= ~DS1307_CLOCK_12HR_MODE;
    buff.hours |= (mode & DS1307_CLOCK_12HR_MODE);

    /* Commit recovered time and new initialisation settings to RTC. */
    ds1307_set_rtc_from_buff();

    /* Set control register with square wave output status, frequency */
    /* and disabled output level configuration. */
    ds1307_set_reg_byte(DS1307_CONTROL, mode & 0x93);

    /* Configure interrupt processing hardware for when RTC square wave */
    /* output is used as low-active tick trigger for external interrupt 1. */
    if (mode & DS1307_SQW_OP_TO_EXT_INT_1)
    {
        /* Set external interrupt 1 port pin (P3^3) as input. */
        INT1 = 1;

        /* Set falling edge trigger for external interrupt 1. */
        IT1 = 1;

        /* Enable external interrupt 1. */
        EX1 = 1;

        /* Enable global interrupt. */
        EA = 1;
    }

    return;
}


/*****************************************************************************
 *
 *                           ds1307_set_reg_byte()
 *
 *****************************************************************************/
void ds1307_set_reg_byte(unsigned char address, unsigned char byte)
{
    /* 
     * Write byte to I2C RTC device at specified address.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C ds1307 RTC device register for write. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_WRITE);
  
    /* Write into word pointer to address required register. */
    i2c_transmit_byte(address);

    /* Transmit data byte to be written, to serial RTC device. */
    i2c_transmit_byte(byte);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return;
}


/*****************************************************************************
 *
 *                           ds1307_get_reg_byte()
 *
 *****************************************************************************/
unsigned char ds1307_get_reg_byte(unsigned char address)
{
    unsigned char byte;

    /* 
     * Read byte from I2C RTC device at specified address.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C ds1307 RTC device for word pointer write. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_WRITE);

    /* Write into word pointer to address required register. */
    i2c_transmit_byte(address);

    /* Transmit I2C restart condition. */
    i2c_transmit_restart();

    /* Select I2C RTC device for read. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_READ);

    /* Retrieve single byte of stored data. (I2C_NAK indicates final */
    /* retrieval of a, possibly extended, reception sequence).       */
    byte = i2c_receive_byte(I2C_NAK);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return(byte);
}


/*****************************************************************************
 *
 *                         ds1307_get_buff_from_rtc()
 *
 *****************************************************************************/
void ds1307_get_buff_from_rtc(void)
{
    /* 
     * Copy to time and date buffer from I2C RTC device registers.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C ds1307 RTC device for word pointer write. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_WRITE);

    /* Write into word pointer to address initial register. */
    i2c_transmit_byte(DS1307_SEC);

    /* Transmit I2C restart condition. */
    i2c_transmit_restart();

    /* Select I2C RTC device for read. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_READ);

    /* Retrieve second data from I2C RTC device. */
    buff.seconds = i2c_receive_byte(I2C_ACK);

    /* Retrieve minute data from I2C RTC device. */
    buff.minutes = i2c_receive_byte(I2C_ACK);

    /* Retrieve hour data from I2C RTC device. */
    buff.hours = i2c_receive_byte(I2C_ACK);

    /* Retrieve day data from I2C RTC device. */
    buff.day = i2c_receive_byte(I2C_ACK);

    /* Retrieve date data from I2C RTC device. */
    buff.date = i2c_receive_byte(I2C_ACK);

    /* Retrieve month data from I2C RTC device. */
    buff.month = i2c_receive_byte(I2C_ACK);

    /* Retrieve year data from I2C RTC device as last in sequence. */
    buff.year = i2c_receive_byte(I2C_NAK);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return;
}


/*****************************************************************************
 *
 *                         ds1307_set_rtc_from_buff()
 *
 *****************************************************************************/
void ds1307_set_rtc_from_buff(void)
{
    /* 
     * Copy to I2C RTC device registers from time and date buffer.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C ds1307 RTC device register for write. */
    i2c_transmit_byte(DS1307_RTC_ID | DS1307_WRITE);
  
    /* Write into word pointer to address initial register. */
    i2c_transmit_byte(DS1307_SEC);

    /* Transmit buffered seconds data to serial RTC device. */
    i2c_transmit_byte(buff.seconds);

    /* Transmit buffered minutes data to serial RTC device. */
    i2c_transmit_byte(buff.minutes);

    /* Transmit buffered hours data to serial RTC device. */
    i2c_transmit_byte(buff.hours);

    /* Transmit buffered day data to serial RTC device. */
    i2c_transmit_byte(buff.day);

    /* Transmit buffered date data to serial RTC device. */
    i2c_transmit_byte(buff.date);

    /* Transmit buffered month data to serial RTC device. */
    i2c_transmit_byte(buff.month);

    /* Transmit buffered year data to serial RTC device. */
    i2c_transmit_byte(buff.year);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return;
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_second()
 *
 *****************************************************************************/
void ds1307_set_buff_second(unsigned char second)
{
    /* Clear lower 7-bits of buffered seconds register, */
    /* whilst preserving clock run bit 7. */
    buff.seconds &= 0x80;

    /* Fill lower 7-bits of buffered seconds register with BCD encoded secs.*/
    buff.seconds |= (0x7F & ds1307_hex_to_BCD(second));

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_second()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_second(void)
{
    /* Return hex encoded BCD value of lower 7-bits of */
    /* buffered seconds register. */
    return(ds1307_BCD_to_hex(0x7F & buff.seconds));
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_minute()
 *
 *****************************************************************************/
void ds1307_set_buff_minute(unsigned char minute)
{
    /* Fill lower 7-bits of buffered minutes register with BCD encoded mins.*/
    buff.minutes = (0x7F & ds1307_hex_to_BCD(minute));

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_minute()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_minute(void)
{
    /* Return hex encoded BCD value of lower 7-bits of */
    /* buffered minutes register. */
    return(ds1307_BCD_to_hex(0x7F & buff.minutes));
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_hour()
 *
 *****************************************************************************/
void ds1307_set_buff_hour(unsigned char hour)
{
    /* Check 12 or 24-hour mode bit 6 in buffered hour register for mode. */
    if (0x40 & buff.hours)
    {
        /* 12-hour mode. */

        /* Clear lower 5-bits of buffered hours register, */
        /* whilst preserving 12 or 24-hour bit 6 and am-pm bit 5. */
        buff.hours &= 0x60;

        /* Fill lower 5-bits of buffered hours register */
        /* with BCD encoded hours. */
        buff.hours |= (0x1F & ds1307_hex_to_BCD(hour));
    }
    else /* 24-hour mode. */
    {
        /* Clear lower 6-bits of buffered hours register, */
        /* whilst preserving 12 or 24-hour bit 6. */
        buff.hours &= 0x40;

        /* Fill lower 6-bits of buffered hours register */
        /* with BCD encoded hours. */
        buff.hours |= (0x3F & ds1307_hex_to_BCD(hour));
    }

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_hour()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_hour(void)
{
    /* Check 12 or 24-hour mode bit 6 in buffered hour register for mode. */
    if (0x40 & buff.hours)
    {
        /* 12-hour mode. */

        /* Return hex encoded BCD value of lower 5-bits of */
        /* buffered hours register. */
        return(ds1307_BCD_to_hex(0x1F & buff.hours));
    }
    
    /* else 24-hour mode. */

    /* Return hex encoded BCD value of lower 6-bits of */
    /* buffered hours register. */
    return(ds1307_BCD_to_hex(0x3F & buff.hours));
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_am_pm()
 *
 *****************************************************************************/
void ds1307_set_buff_am_pm(bit pm_bit)
{
    if (pm_bit)
    {
        /* Set am-pm bit 5 of buffered hours register, as PM requested. */
        buff.hours |= 0x20;
    }
    else
    {
        /* Reset am-pm bit 5 of buffered hours register, as AM requested. */
        buff.hours &= ~0x20;
    }

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_am_pm()
 *
 *****************************************************************************/
bit ds1307_get_buff_am_pm(void)
{
    /* Return am-pm bit 5 of buffered hours register. */
    return((bit)(0x20 & buff.hours));
}


/*****************************************************************************
 *
 *                        ds1307_get_buff_am_pm_as_text()
 *
 *****************************************************************************/
char * ds1307_get_buff_am_pm_as_text(void)
{
    /* Return value of bit 5 of buffered hours register, */
    /* as a pointer to a text string representing am or pm. */
    return(am_pm_as_text[(0x20 & buff.hours) >> 5]);
}


/*****************************************************************************
 *
 *                             ds1307_set_buff_day()
 *
 *****************************************************************************/
void ds1307_set_buff_day(unsigned char day)
{
    /* Write day to lower 3-bits of buffered day register. */
    buff.day = (0x07 & day);

    return;
}


/*****************************************************************************
 *
 *                             ds1307_get_buff_day()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_day(void)
{
    /* Return value of lower 3-bits of buffered day register. */
    return(0x07 & buff.day);
}


/*****************************************************************************
 *
 *                         ds1307_get_buff_day_as_text()
 *
 *****************************************************************************/
char * ds1307_get_buff_day_as_text(void)
{
    /* Return value of lower 3-bits of buffered day register, */
    /* as a pointer to a text string representing each day. */
    return(day_as_text[(0x07 & buff.day)]);
}


/*****************************************************************************
 *
 *                             ds1307_set_buff_date()
 *
 *****************************************************************************/
void ds1307_set_buff_date(unsigned char date)
{
    /* Fill lower 6-bits of buffered date register with BCD encoded date. */
    buff.date = (0x3F & ds1307_hex_to_BCD(date));

    return;
}


/*****************************************************************************
 *
 *                             ds1307_get_buff_date()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_date(void)
{
    /* Return hex encoded BCD value of lower 6-bits of */
    /* buffered date register. */
    return(ds1307_BCD_to_hex(0x3F & buff.date));
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_month()
 *
 *****************************************************************************/
void ds1307_set_buff_month(unsigned char month)
{
    /* Fill lower 5-bits of buffered months register with months in BCD. */
    buff.month = (0x1F & ds1307_hex_to_BCD(month));

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_month()
 *
 *****************************************************************************/
unsigned char ds1307_get_buff_month(void)
{
    /* Return hex encoded BCD value of lower 5-bits of */
    /* buffered months register. */
    return(ds1307_BCD_to_hex(0x1F & buff.month));
}


/*****************************************************************************
 *
 *                        ds1307_get_buff_month_as_text()
 *
 *****************************************************************************/
char * ds1307_get_buff_month_as_text(void)
{
    /* Return hex encoded BCD value of lower 5-bits of buffered months /*
    /* register, as a pointer to a text string representing each month. */
    return(month_as_text[ds1307_BCD_to_hex(0x1F & buff.month)]);
}


/*****************************************************************************
 *
 *                            ds1307_set_buff_year()
 *
 *****************************************************************************/
void ds1307_set_buff_year(unsigned int year)
{
    /* Fill buffered years register with 2 least significant years in BCD.*/
    buff.year = ds1307_hex_to_BCD((unsigned char)(year - 2000u));

    return;
}


/*****************************************************************************
 *
 *                            ds1307_get_buff_year()
 *
 *****************************************************************************/
unsigned int ds1307_get_buff_year(void)
{
    /* Return hex encoded BCD value of buffered years register, 
    /* adjusted for 2nd millennium. */
    return(2000u + ds1307_BCD_to_hex(buff.year));
}


/** @} */
