/**
 * \addtogroup DS1307
 * @{
 */

/*****************************************************************************/
/**
 * \file DS1307.h
 * DS1307 real-time clock routines for the 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 01-05-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef DS1307_H
#define DS1307_H

#include "I2C.h"


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/*
 * DS1307 real-time clock i2c device identification code.
 */
#define DS1307_RTC_ID       0xD0

/*
 * DS1307 real-time clock i2c device read code.
 */
#define DS1307_READ         0x01

/*
 * DS1307 real-time clock i2c device write code.
 */
#define DS1307_WRITE        0x00

/*
 * DS1307 real-time clock seconds register address.
 */
#define DS1307_SEC          0x00
	
/*
 * DS1307 real-time clock minutes register address.
 */
#define	DS1307_MIN          0x01
	
/*
 * DS1307 real-time clock hours register address.
 */
#define DS1307_HOUR         0x02
	
/*
 * DS1307 real-time clock day of week register address.
 */
#define DS1307_DAY          0x03
	
/*
 * DS1307 real-time clock day of month register address.
 */
#define DS1307_DATE         0x04
	
/*
 * DS1307 real-time clock month register address.
 */
#define DS1307_MONTH        0x05
	
/*
 * DS1307 real-time clock year register address.
 */
#define DS1307_YEAR         0x06
	
/*
 * DS1307 real-time clock control register address.
 */
#define DS1307_CONTROL      0x07

/*
 * DS1307 real-time clock non-volatile RAM registers starting address.
 */
#define DS1307_NVRAM       0x08

/** 
 * DS1307 real-time clock non-volatile RAM access macro.
 * \param addr The \b unsigned \b char specifying which nvram register to set.
 * \param byte The \b unsigned \b char specifying the byte data to set.
 * \return -
 * \hideinitializer
 */
#define DS1307_SET_NVRAM_BYTE(addr, byte)                                 \
                        ds1307_set_reg_byte(DS1307_NVRAM + (addr), (byte))

/** 
 * DS1307 real-time clock non-volatile RAM access macro.
 * \param addr The \b unsigned \b char specifying the nvram register to get.
 * \return The \b unsigned \b char representing the byte data retrieved.
 * \hideinitializer
 */
#define DS1307_GET_NVRAM_BYTE(addr)                                       \
                                ds1307_get_reg_byte(DS1307_NVRAM + (addr))

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_SUN          0x01

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_MON          0x02

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_TUE          0x03

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_WED          0x04

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_THU          0x05

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_FRI          0x06

/** 
 * DS1307 real-time clock day of the week define.
 * Pass to ds1307_set_buff_day() to select day of week.
 * \hideinitializer
 */
#define DS1307_SAT          0x07

/** 
 * DS1307 real-time clock am/pm define.
 * Pass to ds1307_set_buff_am_pm() to select am.
 * \hideinitializer
 */
#define DS1307_AM           0x00

/** 
 * DS1307 real-time clock am/pm define.
 * Pass to ds1307_set_buff_am_pm() to select pm.
 * \hideinitializer
 */
#define DS1307_PM           0x01

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to enable clock oscillator.
 * Use of this define is mutually exclusive with DS1307_CLOCK_HALT.
 * \hideinitializer
 */
#define DS1307_CLOCK_RUN              0x00

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to disable clock oscillator.
 * Use of this define is mutually exclusive with DS1307_CLOCK_RUN.
 * \hideinitializer
 */
#define DS1307_CLOCK_HALT             0x08

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to enable 12-hour clock mode.
 * Use of this define is mutually exclusive with DS1307_CLOCK_24HR_MODE.
 * \hideinitializer
 */
#define DS1307_CLOCK_12HR_MODE        0x40

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to enable 24-hour clock mode.
 * Use of this define is mutually exclusive with DS1307_CLOCK_12HR_MODE.
 * \hideinitializer
 */
#define DS1307_CLOCK_24HR_MODE        0x00

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to enable RTC square wave output.
 * Use of this define is mutually exclusive with DS1307_SQW_OP_OFF.
 * \hideinitializer
 */
#define DS1307_SQW_OP_ON              0x10

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to disable RTC square wave output.
 * Use of this define is mutually exclusive with DS1307_SQW_OP_ON,
 * and it must be used in combination with either DS1307_SQW_OP_OFF_LVL_0
 * or DS1307_SQW_OP_OFF_LVL_1.
 * \hideinitializer
 */
#define DS1307_SQW_OP_OFF             0x00

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to select 1Hz square wave output frequency.
 * Use of this define is mutually exclusive with each of 
 * DS1307_SQW_OP_FREQ_4096HZ, DS1307_SQW_OP_FREQ_8192HZ and
 * DS1307_SQW_OP_FREQ_32768HZ.
 * \hideinitializer
 */
#define DS1307_SQW_OP_FREQ_1HZ        0x00

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to select 4.096kHz square wave output frequency.
 * Use of this define is mutually exclusive with each of 
 * DS1307_SQW_OP_FREQ_1HZ, DS1307_SQW_OP_FREQ_8192HZ and
 * DS1307_SQW_OP_FREQ_32768HZ.
 * \hideinitializer
 */
#define DS1307_SQW_OP_FREQ_4096HZ     0x01

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to select 8.192kHz square wave output frequency.
 * Use of this define is mutually exclusive with each of 
 * DS1307_SQW_OP_FREQ_1HZ, DS1307_SQW_OP_FREQ_4096HZ and
 * DS1307_SQW_OP_FREQ_32768HZ.
 * \hideinitializer
 */
#define DS1307_SQW_OP_FREQ_8192HZ     0x02

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to select 32.768kHz square wave output frequency.
 * Use of this define is mutually exclusive with each of 
 * DS1307_SQW_OP_FREQ_1HZ, DS1307_SQW_OP_FREQ_4096HZ and
 * DS1307_SQW_OP_FREQ_8192HZ.
 * \hideinitializer
 */
#define DS1307_SQW_OP_FREQ_32768HZ    0x03

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to force disabled square wave output to low level.
 * This define must be used in combination with DS1307_SQW_OP_OFF.
 * \hideinitializer
 */
#define DS1307_SQW_OP_OFF_LVL_0       0x00

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to force disabled square wave output to high level.
 * This define must be used in combination with DS1307_SQW_OP_OFF.
 * \hideinitializer
 */
#define DS1307_SQW_OP_OFF_LVL_1       0x80

/** 
 * DS1307 real-time clock square wave output pin access macro.
 * Call the DS1307_SET_SQW_OP_OFF_LVL_0() macro to set the disabled square
 * wave oscillator's output pin to low level.
 * \hideinitializer
 */
#define DS1307_SET_SQW_OP_OFF_LVL_0()                                      \
                ds1307_set_reg_byte(DS1307_CONTROL, DS1307_SQW_OP_OFF_LVL_0)

/** 
 * DS1307 real-time clock square wave output pin access macro.
 * Call the DS1307_SET_SQW_OP_OFF_LVL_1() macro to set the disabled square
 * wave oscillator's output pin to high level.
 * \hideinitializer
 */
#define DS1307_SET_SQW_OP_OFF_LVL_1()                                      \
                ds1307_set_reg_byte(DS1307_CONTROL, DS1307_SQW_OP_OFF_LVL_1)

/** 
 * DS1307 real-time clock control define.
 * Pass to ds1307_init() to enable interrupt processing when square wave
 * output is connected to external interrupt 1.
 * This define may be used in combination with DS1307_SQW_OP_ON.
 * \hideinitializer
 */
#define DS1307_SQW_OP_TO_EXT_INT_1    0x04


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * DS1307 real-time clock initialisation function.
 * Initialises and starts the DS1307 real-time clock hardware.
 * Must be called once at program commencement to start the clock, 
 * select 12 or 24-hour mode, and possibly set-up square wave output
 * for periodic microntroller interrupt. On return, the RTC time and
 * date are held in the local buffers.
 * \param mode The \b unsigned \b char containing RTC setup configuration.
 * \return -
 */
extern void ds1307_init(unsigned char mode);

/** 
 * DS1307 real-time clock set register byte function.
 * 
 * \param address The \b unsigned \b char specifying which of the 64 
 * register bytes to set.
 * \param byte The \b unsigned \b char containing register byte data to set.
 * \return -
 */
extern void ds1307_set_reg_byte(unsigned char address, unsigned char byte);

/** 
 * DS1307 real-time clock get register byte function.
 * 
 * \param address The \b unsigned \b char specifying which of the 64 
 * register bytes to set.
 * \return The \b unsigned \b char containing register data byte retrieved.
 */
extern unsigned char ds1307_get_reg_byte(unsigned char address);

/** 
 * DS1307 real-time clock time, day and date capture function.
 * Call to copy the RTC time, day and date to the local buffer.
 * \param -
 * \return -
 */
extern void ds1307_get_buff_from_rtc(void);

/** 
 * DS1307 real-time clock time, day and date buffer commit function.
 * Call to copy the locally buffered time, day and date to the RTC.
 * \param -
 * \return -
 */
extern void ds1307_set_rtc_from_buff(void);

/** 
 * DS1307 real-time clock buffered seconds set function.
 * \param second The \b unsigned \b char specifying the second (0..59) to set.
 * \return -
 */
extern void ds1307_set_buff_second(unsigned char second);

/** 
 * DS1307 real-time clock buffered seconds get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered second (0..59).
 */
extern unsigned char ds1307_get_buff_second(void);

/** 
 * DS1307 real-time clock buffered minutes set function.
 * \param minute The \b unsigned \b char specifying the minute (0..59) to set.
 * \return -
 */
extern void ds1307_set_buff_minute(unsigned char minute);

/** 
 * DS1307 real-time clock buffered minutes get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered minute (0..59).
 */
extern unsigned char ds1307_get_buff_minute(void);

/** 
 * DS1307 real-time clock buffered hours set function.
 * \param hour The \b unsigned \b char specifying the hour (1..12 | 0..23)
 * to set.
 * \return -
 */
extern void ds1307_set_buff_hour(unsigned char hour);

/** 
 * DS1307 real-time clock buffered hours get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered hour
 * (1..12 | 0..23).
 */
extern unsigned char ds1307_get_buff_hour(void);

/** 
 * DS1307 real-time clock buffered am pm set function.
 * \param pm_bit The \b bit indicating pm bit status to set.
 * \return -
 */
extern void ds1307_set_buff_am_pm(bit pm_bit);

/** 
 * DS1307 real-time clock buffered am pm get function.
 * \param -
 * \return The \b bit indicating pm bit status.
 */
extern bit ds1307_get_buff_am_pm(void);

/** 
 * DS1307 real-time clock buffered am pm get as text function.
 * \param -
 * \return The \b char \b * to text string indicating am pm bit status.
 */
extern char * ds1307_get_buff_am_pm_as_text(void);

/** 
 * DS1307 real-time clock buffered days set function.
 * \param day The \b unsigned \b char specifying the day of the week (1..7)
 * to set.
 * \return -
 */
extern void ds1307_set_buff_day(unsigned char day);

/** 
 * DS1307 real-time clock buffered days get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered day of
 * the week (1..7).
 */
extern unsigned char ds1307_get_buff_day(void);

/** 
 * DS1307 real-time clock buffered days textual get function.
 * \param -
 * \return The \b unsigned \b char \b * pointing to a textual 
 * representation of the day of the week.
 */
extern char * ds1307_get_buff_day_as_text(void);

/** 
 * DS1307 real-time clock buffered date set function.
 * \param date The \b unsigned \b char specifying the day of the
 * month (1..31) to set.
 * \return -
 */
extern void ds1307_set_buff_date(unsigned char date);

/** 
 * DS1307 real-time clock buffered date get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered day
 * of the month (1..31).
 */
extern unsigned char ds1307_get_buff_date(void);

/** 
 * DS1307 real-time clock buffered months set function.
 * \param month The \b unsigned \b char specifying the month (1..12) to set.
 * \return -
 */
extern void ds1307_set_buff_month(unsigned char month);

/** 
 * DS1307 real-time clock buffered months get function.
 * \param -
 * \return The \b unsigned \b char representing the buffered month (1..12).
 */
extern unsigned char ds1307_get_buff_month(void);

/** 
 * DS1307 real-time clock buffered months textual get function.
 * \param -
 * \return The \b unsigned \b char \b * pointing to a textual 
 * representation of the month.
 */
extern char * ds1307_get_buff_month_as_text(void);

/** 
 * DS1307 real-time clock buffered years set function.
 * \param year The \b unsigned \b int specifying the year to set (00..99).
 * \return -
 */
extern void ds1307_set_buff_year(unsigned int year);

/** 
 * DS1307 real-time clock buffered years get function.
 * \param -
 * \return The \b unsigned \b int representing the buffered 
 * year (2000..2099).
 */
extern unsigned int ds1307_get_buff_year(void);


#endif

/** @} */
