/**
 * \addtogroup I2C
 * @{
 */

/*****************************************************************************/
/**
 * \file I2C.c
 * 100kHz I2C master routines for 8051 and Keil C51.
 * \author Copyright (c) 2009, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 03-12-09
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#include "I2C.h"


/*****************************************************************************
 *
 *                         Private Function Prototypes
 *
 *****************************************************************************/
static void i2c_delay_5us(void);


/*****************************************************************************
 *
 *                         Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                               i2c_delay_5us()
 *
 *****************************************************************************/
static void i2c_delay_5us(void)
{
#if (I2C_XTAL <= 3686400)

    /*
     * Routine contents for an approximately < 13us function call delay, using
     * a 3.6864MHz or near, yet slower xtal.
     */

#elif (I2C_XTAL <= 11059200)

    /*
     * Routine contents for an approximately 6.5us function call delay, using
     * a 11.0592MHz or near, yet slower xtal.
     */
    _nop_(); _nop_();

  #if (I2C_X2_MODE == 1)

    _nop_(); _nop_(); _nop_(); _nop_();

  #endif

#elif (I2C_XTAL <= 13830600)

    /*
     * Routine contents for an approximately 5us function call delay, using
     * a 13.8306MHz or near, yet slower xtal.
     */
    _nop_(); _nop_(); _nop_();

  #if (I2C_X2_MODE == 1)

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

  #endif

#elif (I2C_XTAL <= 18432000)

    /*
     * Routine contents for an approximately 5us function call delay, using
     * a 18.432MHz or near, yet slower xtal.
     */
    _nop_(); _nop_(); _nop_(); _nop_();

  #if (I2C_X2_MODE == 1)

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

  #endif

#elif (I2C_XTAL <= 22118400)

    /*
     * Routine contents for an approximately 5us function call delay, using
     * a 22.1184MHz or near, yet slower xtal.
     */
    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

  #if (I2C_X2_MODE == 1)

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

    _nop_();

  #endif

#elif (I2C_XTAL <= 40000000)

    /*
     * Routine contents for an approximately 5us function call delay, using
     * a 40.0MHz or near, yet slower xtal.
     */
    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

  #if (I2C_X2_MODE == 1)

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

    _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_(); _nop_();

    _nop_();

  #endif

#else

    #error Crystal frequency selection exceeds program version capabilities.

#endif

    return;
}


/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                            i2c_transmit_start()
 *
 *****************************************************************************/
void i2c_transmit_start(void)
{
    /* 
     * Transmit I2C start condition sequence.
     */

    SDA = 0;

    /* Delay >= 4us. */
    i2c_delay_5us();

    SCL = 0;

    return;
}


/*****************************************************************************
 *
 *                           i2c_transmit_restart()
 *
 *****************************************************************************/
void i2c_transmit_restart(void)
{
    /* 
     * Transmit I2C restart condition sequence.
     */

    SDA = 1;

    SCL = 1;

    /* Delay >= 4.7us. */
    i2c_delay_5us();

    SDA = 0;

    /* Delay >= 4us. */
    i2c_delay_5us();

    SCL = 0;

    return;
}


/*****************************************************************************
 *
 *                            i2c_transmit_stop()
 *
 *****************************************************************************/
void i2c_transmit_stop(void)
{
    /* 
     * Transmit I2C stop condition sequence.
     */

    SDA = 0;

    SCL = 1;

    /* Delay >= 4us. */
    i2c_delay_5us();

    SDA = 1;

    /* Delay >= 4.7us. */
    i2c_delay_5us();

    return;
}


/*****************************************************************************
 *
 *                             i2c_transmit_reset()
 *
 *****************************************************************************/
void i2c_transmit_reset(void)
{
    unsigned char i;

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    SDA = 1;

    /* Send 9 successive clock pulses with data line held high. */
    for(i = 9u; i > 0u; i--)
    {
        /* Delay >= 250ns. */

        SCL = 1;

        /* Delay >= 4us. */
        i2c_delay_5us();

        SCL = 0;
    }

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return;
}


/*****************************************************************************
 *
 *                               i2c_transmit_byte()
 *
 *****************************************************************************/
void i2c_transmit_byte(unsigned char byte)
{
    unsigned char i;

    /* 
     * Transmit byte bit-by-bit from MSB to LSB.
     */
    for(i = 8u; i > 0u; i--)
    {
        /* Is a bit set in byte? */
        if(byte & (1 << (i - 1)))
        {
            /* Bit set so make data output pin high. */
            SDA = 1;
        }
        else
        {
            /* Bit clear so make data output pin low. */
            SDA = 0;
        }

        /* Delay >= 250ns. */

        SCL = 1;

        /* Delay >= 4us. */
        i2c_delay_5us();

        SCL = 0;
    }

    /* Set data output line as an input pin. */
    SDA = 1;

    SCL = 1;

    /* Delay >= 4us. */
    i2c_delay_5us();

    /* Wait for slave to acknowledge. */
    while(SDA) continue;

    SCL = 0;

    /* Delay >= 4.7us. */
    i2c_delay_5us();

    return;
}



/*****************************************************************************
 *
 *                    i2c_transmit_byte_then_test_for_ack()
 *
 *****************************************************************************/
bit i2c_transmit_byte_then_test_for_ack(unsigned char byte)
{
    unsigned char i;
    bit reply = I2C_NAK;

    /* 
     * Transmit byte bit-by-bit from MSB to LSB. 
     */
    for(i = 8u; i > 0u; i--)
    {
        /* Is a bit set in byte? */
        if(byte & (1 << (i - 1)))
        {
            /* Bit set so make data output pin high. */
            SDA = 1;
        }
        else
        {
            /* Bit clear so make data output pin low. */
            SDA = 0;
        }

        /* Delay >= 250ns. */

        SCL = 1;

        /* Delay >= 4us. */
        i2c_delay_5us();

        SCL = 0;
    }

    /* Set data output line as an input pin. */
    SDA = 1;

    SCL = 1;

    /* Delay >= 4us. */
    i2c_delay_5us();

    /* 
     * Look for slave device acknowledge reply by testing whether SDA held
     * low during acknowledge clock pulse.
     */
    if(!SDA)
    {
        /* Slave device did acknowledge. */
        reply = I2C_ACK;
    }

    SCL = 0;

    /* Delay >= 4.7us. */
    i2c_delay_5us();

    return(reply);
}


/*****************************************************************************
 *
 *                             i2c_receive_byte()
 *
 *****************************************************************************/
unsigned char i2c_receive_byte(bit final)
{
    unsigned char i;
    unsigned char byte = 0x00;

    /* Set data output line as an input pin. */
    SDA = 1;

    /* 
     * Receive byte bit-by-bit from MSB to LSB.
     */
    for(i = 8u; i > 0u; i--)
    {
        SCL = 1;

        /* Delay >= 4us. */
        i2c_delay_5us();

        if(SDA)
        {
            /* Incoming bit is high so set corresponding bit in byte. */
            byte |= (1 << (i - 1));
        } 
        else 
        {
            /* Incoming bit is low so clear corresponding bit in byte. */
            byte &= ~(1 << (i - 1));
        }

        SCL = 0;

        /* Delay >= 4.7us. */
        i2c_delay_5us();
    }

    /* Send I2C_ACK or I2C_NAK continuation bit to transmitting slave. */
    SDA = final;

    SCL = 1;

    /* Delay >= 4us. */
    i2c_delay_5us();

    SCL = 0;

    /* Delay >= 4.7us. */
    i2c_delay_5us();

    return byte;
}

/** @} */
