/**
 * \addtogroup I2C
 * @{
 */

/*****************************************************************************/
/**
 * \file I2C.h
 * 100kHz I2C master routines for 8051 and Keil C51.
 * \author Copyright (c) 2009, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 03-12-09
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef I2C_H
#define I2C_H

#include "Reg51.h"
#include <intrins.h>


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/** 
 * Hardware crystal frequency in Hz.
 * Adjust \b I2C_XTAL to match hardware.
 * \hideinitializer
 */
#define I2C_XTAL          18432000

/** 
 * X2 mode support binary switch.
 * Set \b I2C_X2_MODE when CPU X2 clock mode has been elsewhere
 * engaged.
 * \hideinitializer
 */
#define I2C_X2_MODE       0

/** 
 * Serial data input/ output line.
 * Adjust \b SDA to match hardware.
 * \hideinitializer
 */
sbit SDA = P1^3;

/** 
 * Serial clock output line.
 * Adjust \b SCL to match hardware.
 * \hideinitializer
 */
sbit SCL = P1^4;

/** 
 * I2C data reception control define.
 * When passed to i2c_receive_byte(), indicates further reception follows.
 * Returned by i2c_transmit_byte_then_test_for_ack() when slave acknowledges.
 * \hideinitializer
 */
#define I2C_ACK       0

/** 
 * I2C data reception control define.
 * When passed to i2c_receive_byte(), indicates no further reception follows.
 * Returned by i2c_transmit_byte_then_test_for_ack() when slave negative acks.
 * \hideinitializer
 */
#define I2C_NAK       1


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * I2C start condition output function.
 * \param -
 * \return -
 */
extern void i2c_transmit_start(void);

/** 
 * I2C restart condition output function.
 * \param -
 * \return -
 */
extern void i2c_transmit_restart(void);

/** 
 * I2C stop condition output function.
 * \param -
 * \return -
 */
extern void i2c_transmit_stop(void);

/** 
 * I2C protocol reset slaves function.
 * Send software reset sequence so as to I2C-protocol-reset all connected
 * slave devices. Useful after a protocol interruption, power loss or
 * system reset.
 * \param -
 * \return -
 */
extern void i2c_transmit_reset(void);

/** 
 * I2C byte output function.
 * Transmits the given byte via the I2C bus, then blocks to extend for any
 * slow-to-acknowledge slave devices.
 * \param byte The \b unsigned \b char to be transmitted on the I2C bus.
 * \return -
 */
extern void i2c_transmit_byte(unsigned char byte);

/** 
 * I2C byte output function with slave reply test.
 * Transmits the given byte via the I2C bus, then tests for acknowledge
 * or null-acknowledge from byte receiving slave device.
 * \param byte The \b unsigned \b char to be transmitted on the I2C bus.
 * \return The \b bit indicating either \b I2C_ACK or \b I2C_NAK slave response.
 */
extern bit i2c_transmit_byte_then_test_for_ack(unsigned char byte);

/** 
 * I2C byte input function.
 * \param final The \b I2C_ACK or \b I2C_NAK bit that respectively indicates whether
 * further reception operations will follow, or if this was the last.
 * \return The \b unsigned \b char read from the I2C bus.
 */
extern unsigned char i2c_receive_byte(bit final);

#endif

/** @} */
